#pragma once

const long MAX_LVITEMLEN = 128;

// Class:   CLVItemData 
// Purpose: Simple class that represents data contained in a list control item.
//          An instance of this class is created for every item in the list control.
// 
// Methods:
//      GetListViewItemInfo(...) -- Called by CLVSelStateDlg::OnLvnGetdispinfoList
//      SetSelectedState(...) -- Called by CLVSelStateDlg::OnNMClickList
class CLVItemData : public CLockableCS 
{
public:

    enum { LVID_COLUMN_ONE = 0, LVID_COLUMN_TWO = 1, LVID_COLUMN_THREE = 2 };

    CLVItemData( LPCTSTR szColOne, LPCTSTR szColTwo, LPCTSTR szColThree)
        : m_sColumn0ne( szColOne )
        , m_sColumnTwo( szColTwo )
        , m_sColumnThree( szColThree )
        , m_uSelected( TRUE )
    {
    }

    ~CLVItemData()
    {
    }

// Operations
public:
 
    // Method:  GetListViewItemInfo
    // Purpose: Called by CLVSelStateDlg::OnLvnGetdispinfoList
    //          Used to supply the display data for the virtual
    //          list control
    //
    void GetListViewItemInfo( LPLVITEM pItem )
    {
        // Thread safe locking
        CAutoLockT<CLVItemData> lock(this);

        if ((pItem->mask & LVIF_TEXT) == LVIF_TEXT) 
        {
	        switch( pItem->iSubItem )
	        {
	        case LVID_COLUMN_ONE:
                StringCchCopy( pItem->pszText, MAX_LVITEMLEN, m_sColumn0ne );
		        break;
	        case LVID_COLUMN_TWO:

                m_sColumnTwo = m_uSelected ? _T("Checked") : _T("Unchecked");
                StringCchCopy( pItem->pszText, MAX_LVITEMLEN, m_sColumnTwo );
		        break;
	        case LVID_COLUMN_THREE: 
                StringCchCopy( pItem->pszText, MAX_LVITEMLEN, m_sColumnThree );
		        break;
            default:
                ASSERT( 0 );
                StringCchCopy( pItem->pszText, MAX_LVITEMLEN, _T("") );
                TRACE( _T("Invalid Column Index in CLVItemData::GetListViewString() - %d\n"), pItem->iSubItem );
	        }
        }

      	if ((pItem->mask & LVIF_IMAGE) == LVIF_IMAGE) 
        {
            switch( pItem->iSubItem )
            {
            case 0: // Column one
                // Only interested in the first column
                // uSelected has the same index as the image list
                // m_uSelected = iImage = 0 == unchecked
                // m_uSelected = iImage = 0 == checked
                pItem->iImage = m_uSelected; 
                break;
            default:
                // For now we are only interested in the first column
                // as we don't have any images in the 2nd and 3rd columns
                break;
            }
        }
    }

    // Method:  SetSelectedState
    // Purpose: Called by CLVSelStateDlg::OnNMClickList
    //          when a user has clicked on list control item.  This method
    //          currently changes the check box state when the 1st column
    //          icon has been clicked.
    void SetSelectedState( UINT uFlags, LPNMLISTVIEW pnmListView )
    {
        // Thread safe locking
        CAutoLockT<CLVItemData> lock(this);

        switch( pnmListView->iSubItem )
        {
        case 0: // Column one
            // We're clicking on the check box icon, so toggle its state    
            if ( (uFlags & TVHT_ONITEMICON) == TVHT_ONITEMICON )
                m_uSelected = !m_uSelected;
            break;
        // case n: // add other column handling here
        default:
            // For now we are only interested in the first column
            // as we don't have any images in the 2nd and 3rd columns
            break;
        }
    }

// Attributes
private:
    CString m_sColumn0ne;
    CString m_sColumnTwo;
    CString m_sColumnThree;
    UINT    m_uSelected;
};

// Turn a std::vector into a lockable vector
class CLVItemDataList 
    : public std::vector< CLVItemData* >
    , public CLockableCS 
{
public:
	CLVItemDataList() {}
	~CLVItemDataList() {}
};

